import "./public-path"
import Vue from 'vue'
import Design from '@/views/Design.vue'
import ElementUI from 'element-ui'
import 'element-ui/packages/theme-chalk/src/index.scss'
import registerGlobComp from '@/components'
import { WORKFLOW_DESIGN_APP_PROPS_KEY, WORKFLOW_DESIGN_APP_EMIT_KEY } from './constants'
import { setupStore } from './store'
import { useMicroAppStoreWithOut } from './store/modules/microApp'
import { createPinia } from 'pinia'
import { isEmpty } from 'lodash-es'
const store = createPinia()
let instance = null
function render(props = {}) {
  const { container, mountApp } = props
  instance = new Vue({
    pinia: store,
    render: h => h(getComponent(mountApp), { props })
  }).$mount(container ? container.querySelector('#app') : '#app')
}

// fixme: 开发阶段使用,生产环境可以注释掉(目前已经完全适配微应用,必须在微应用环境下运行)
/*if (!window.__POWERED_BY_QIANKUN__) {
  bootstrap().then(() => render())
}*/

export async function bootstrap() {
  setupStore(Vue)
  Vue.use(ElementUI, { size: 'medium' }).use(registerGlobComp)
  Vue.config.productionTip = false
}

export async function mount(props) {
  const { microData } = props
  Vue.prototype.$qiankunProps = props
  Vue.prototype.$setGlobalState = setGlState(props)
  Vue.prototype.$useMessage = microData.useMessage
  window.$glob.ticket = microData.token
  render(props)
  const microAppStore = useMicroAppStoreWithOut()
  microAppStore.setMainData(microData)
  props.onGlobalStateChange((newState, prev) => {
    const { [WORKFLOW_DESIGN_APP_PROPS_KEY]: workflowDesignProps } = newState
    !isEmpty(workflowDesignProps) && microAppStore.setMainState(workflowDesignProps)
  }, true)
  const ref = instance.$children[0]
  instance.$setGlobalState({
    [instance.$children[0].$options.name]: {
      getRef: () => ref
    }
  })
}

export async function unmount() {
  instance.$destroy()
  instance.$el.innerHTML = ''
  instance = null
}

/** 处理传递微应用全局参数 */
const setGlState = (props) => (state) => props.setGlobalState({
  [WORKFLOW_DESIGN_APP_EMIT_KEY]: {
    ...state
  }
})

/** 处理查找渲染组件 */
const requireComponent = require.context('./views', true, /\.vue$/);
function getComponent(componentName = '') {
  const normalizedComponentName = componentName.toLowerCase();
  // 遍历 components 目录下的每个文件
  for (const fileName of requireComponent.keys()) {
    const componentConfig = requireComponent(fileName);
    const component = componentConfig.default || componentConfig;
    // 检查组件名是否匹配
    if (component.name && component.name.toLowerCase() === normalizedComponentName) {
      return component;
    }
  }
  return Design;
}
