import axios from 'axios'
import qs from 'qs'
import { checkUrl } from './index'
import { useMicroAppStoreWithOut } from '@/store/modules/microApp'
const microAppStore = useMicroAppStoreWithOut()
const createMessage = microAppStore.getMainData.$useMessage?.createMessage
window.$glob = {
  url: process.env.VUE_APP_BASE_API,
  ticket: '',
  query: {},
  header: {}
}
axios.defaults.timeout = 30000
axios.defaults.validateStatus = function (status) {
  return status >= 200 && status <= 500
}
axios.interceptors.request.use(config => {
  if (!checkUrl(config.url)) config.url = window.$glob.url + config.url
  // 认证方案:使用Bearer
  window.$glob.ticket &&
  (config.headers['Authorization'] = `Bearer ${window.$glob.ticket}`)
  let header = window.$glob.header || {}
  config.headers = Object.assign(config.headers, header)
  let data = window.$glob.query || {}
  let key
  const type = config.method
  if (['get', 'delete'].includes(config.method)) {
    key = "params"
  } else if (['post', 'put'].includes(config.method)) {
    key = "data"
  }
  if (typeof (config[key]) === 'object') {
    config[key] = Object.assign(config[key] || {}, data)
  }
  const arrayFormat = config.headers.arrayFormat || 'indices'
  if (type === 'post' && config.headers['Content-Type'] === 'application/x-www-form-urlencoded; charset=utf-8') {
    config.data = qs.stringify(config.data, { allowDots: true, arrayFormat: arrayFormat })
  } else if (type === 'get') {
    config.params = qs.stringify(config.params, { allowDots: true, arrayFormat: arrayFormat })
    config.params = qs.parse(config.params)
  }
  return config
}, error => {
  return Promise.reject(error)
})

axios.interceptors.response.use(config => {
  const msg = config.data?.msg || config.data?.message|| '系统未知错误,请反馈给管理员'
  const status = Number(config.status) || 200
  if (status !== 200 || config.data?.code === 1) {
    createMessage.error(msg)
    return Promise.reject(new Error(msg))
  }
  return config.data
}, error => {
  let { msg } = error;
  if (msg === "Network Error") {
    msg = "后端接口连接异常";
  }
  else if (msg.includes("timeout")) {
    msg = "系统接口请求超时";
  }
  else if (msg.includes("Request failed with status code")) {
    msg = "系统接口" + msg.substr(msg.length - 3) + "异常";
  }
  createMessage.error(msg)
  return Promise.reject(new Error(error))
})

export default axios
